const
    fs = require("fs"),
    path = require("path"),
    webpack = require("webpack"),
    merge = require("webpack-merge"),
    CopyWebpackPlugin = require("copy-webpack-plugin"),
    HtmlWebpackPlugin = require("html-webpack-plugin"),
    HtmlWebpackInlineSourcePlugin = require("html-webpack-inline-source-plugin"),
    ExtractTextPlugin = require("extract-text-webpack-plugin"),
    OptimizeCSSPlugin = require("optimize-css-assets-webpack-plugin");

let webpackConfig;

module.exports = function (config) {
    if (!webpackConfig) {
        const
            utils = require("./utils"),
            baseWebpackConfig = require("./webpack.base.conf")(config);


        // generate dist index.html with correct asset hash for caching.
        // you can customize output by editing /index.html
        // see https://github.com/ampedandwired/html-webpack-plugin
        let htmlWebpackPlugins = [],
            configPlugins = [];
        if (Array.isArray(config.build.htmlwebpackplugin)) {
            configPlugins = config.build.htmlwebpackplugin;
        } else {
            configPlugins = [config.build.htmlwebpackplugin || {}];
        }
        configPlugins.forEach(plugin => {
            htmlWebpackPlugins.push(
                new HtmlWebpackPlugin(merge({
                    filename: config.build.index,
                    template: config.build.template,
                    inject: true,
                    minify: {
                        removeComments: true,
                        collapseWhitespace: true/** ,
                         removeAttributeQuotes: true */
                        // more options:
                        // https://github.com/kangax/html-minifier#options-quick-reference
                    },
                    // necessary to consistently work with multiple chunks via CommonsChunkPlugin
                    chunksSortMode: "dependency",
                    inlineSource: ".(css)$"
                }, plugin)));
        });

        webpackConfig = merge(baseWebpackConfig, {
            module: {
                rules: utils.styleLoaders({
                    sourceMap: config.build.productionSourceMap,
                    extract: true
                })
            },
            devtool: config.build.productionSourceMap ? "#source-map" : false,
            output: {
                path: config.build.assetsRoot,
                filename: utils.assetsPath("js/[name].[chunkhash].js"),
                chunkFilename: utils.assetsPath("js/[id].[chunkhash].js")
            },
            plugins: [
                // http://vuejs.github.io/vue-loader/en/workflow/production.html
                new webpack.DefinePlugin({
                    "process.env": config.build.env
                }),
                new webpack.optimize.UglifyJsPlugin({
                    compress: {
                        warnings: false
                    },
                    sourceMap: true
                }),
                // extract css into its own file
                new ExtractTextPlugin({
                    filename: utils.assetsPath("css/[name].[contenthash].css")
                }),
                // Compress extracted CSS. We are using this plugin so that possible
                // duplicated CSS from different components can be deduped.
                new OptimizeCSSPlugin({
                    cssProcessorOptions: {
                        safe: true
                    }
                }),
                new HtmlWebpackInlineSourcePlugin(),
                // keep module.id stable when vender modules does not change
                new webpack.HashedModuleIdsPlugin(),
                // split vendor js into its own file
                new webpack.optimize.CommonsChunkPlugin({
                    name: "vendor",
                    minChunks: function (module/** , count */) {
                        // any required modules inside node_modules are extracted to vendor
                        return (
                            module.resource &&
                            /\.js$/.test(module.resource) &&
                            module.resource.indexOf(
                                path.join(__dirname, "../../../../node_modules")
                            ) === 0
                        );
                    }
                }),
                // extract webpack runtime and module manifest to its own file in order to
                // prevent vendor hash from being updated whenever app bundle is updated
                new webpack.optimize.CommonsChunkPlugin({
                    name: "manifest",
                    chunks: ["vendor"]
                })
            ].concat(htmlWebpackPlugins)
        });

        const staticDir = path.resolve(__dirname, "../../../../static");
        // copy custom static assets if exist
        if (fs.existsSync(staticDir)) {
            webpackConfig.plugins.push(new CopyWebpackPlugin([{
                from: staticDir,
                to: config.build.assetsSubDirectory,
                ignore: [".*"]
            }]));
        }

        if (config.build.productionGzip) {
            const CompressionWebpackPlugin = require("compression-webpack-plugin");

            webpackConfig.plugins.push(
                new CompressionWebpackPlugin({
                    asset: "[path].gz[query]",
                    algorithm: "gzip",
                    test: new RegExp(
                        "\\.(" +
                        config.build.productionGzipExtensions.join("|") +
                        ")$"
                    ),
                    threshold: 10240,
                    minRatio: 0.8
                })
            );
        }

        if (config.build.bundleAnalyzerReport) {
            const BundleAnalyzerPlugin = require("webpack-bundle-analyzer").BundleAnalyzerPlugin;
            webpackConfig.plugins.push(new BundleAnalyzerPlugin());
        }
    }
    return webpackConfig;
};